@@ -1,15 +1,15 @@
 import math
 import os
 
-import gym
+import gymnasium
 import numpy as np
 import pybullet as p
-from gym import spaces
+from gymnasium import spaces
 
 from PyFlyt.core.aviary import Aviary
 
 
-class SimpleWaypointEnv(gym.Env):
+class SimpleWaypointEnv(gymnasium.Env):
     """
     Simple Waypoint Environment
 
@@ -17,24 +17,50 @@ class SimpleWaypointEnv(gym.Env):
 
     The target is a set of `[x, y, z, yaw]` targets in space
 
-    Reward is -(distance from waypoint + angle error) for each timestep,
-    and -100.0 for hitting the ground.
+    For sparse reward setting:
+        Reward is 10.0 for reaching target, 0.0 otherwise
+    For non-sparse reward setting:
+        Reward is -(distance from waypoint + angle error) for each timestep,
+        and -100.0 for hitting the ground.
     """
 
     metadata = {"render_modes": ["human"]}
 
     def __init__(
         self,
-        max_steps=10000,
-        angle_representation="quaternion",
-        num_targets=4,
-        use_yaw_targets=True,
-        goal_reach_distance=0.2,
-        goal_reach_angle=0.1,
-        flight_dome_size=5.0,
+        max_steps: int = 10000,
+        angle_representation: str = "quaternion",
+        num_targets: int = 4,
+        use_yaw_targets: bool = True,
+        goal_reach_distance: float = 0.2,
+        goal_reach_angle: float = 0.1,
+        flight_dome_size: float = 5.0,
+        sparse_reward: bool = True,
+        render_mode: None | str = None,
     ):
+        """__init__.
 
-        """GYM STUFF"""
+        Args:
+            max_steps (int): max_steps of the environment
+            angle_representation (str): can be "euler" or "quaternion"
+            num_targets (int): num_targets
+            use_yaw_targets (bool): use_yaw_targets
+            goal_reach_distance (float): goal_reach_distance
+            goal_reach_angle (float): goal_reach_angle
+            flight_dome_size (float): size of the allowable flying area
+            sparse_reward (bool): sparse_reward setting
+            render_mode (None | str): can be "human" or None
+        """
+
+        if render_mode is not None:
+            assert (
+                render_mode in self.metadata["render_modes"]
+            ), f"Invalid render mode {render_mode}, only `human` allowed."
+            self.enable_render = True
+        else:
+            self.enable_render = False
+
+        """GYMNASIUM STUFF"""
         # observation size increases by 1 for quaternion
         if angle_representation == "euler":
             obs_shape = 15
@@ -58,13 +84,13 @@ class SimpleWaypointEnv(gym.Env):
         self.action_space = spaces.Box(low=low, high=high, dtype=np.float64)
 
         """ ENVIRONMENT CONSTANTS """
-        self.enable_render = False
         self.flight_dome_size = flight_dome_size
+        self.sparse_reward = sparse_reward
         self.max_steps = max_steps
+        self.num_targets = num_targets
         self.use_yaw_targets = use_yaw_targets
         self.goal_reach_distance = goal_reach_distance
         self.goal_reach_angle = goal_reach_angle
-        self.ang_rep = 0
         if angle_representation == "euler":
             self.ang_rep = 0
         elif angle_representation == "quaternion":
@@ -72,38 +98,27 @@ class SimpleWaypointEnv(gym.Env):
         file_dir = os.path.dirname(os.path.realpath(__file__))
         self.targ_obj_dir = os.path.join(file_dir, f"../models/target.urdf")
 
-        """ RUNTIME VARIABLES """
-        self.env = None
+        self.reset()
 
-        # we sample from polar coordinates to generate linear targets
-        self.targets = np.zeros(shape=(num_targets, 3))
-        thts = np.random.uniform(0.0, 2.0 * math.pi, size=(num_targets,))
-        phis = np.random.uniform(0.0, 2.0 * math.pi, size=(num_targets,))
-        for i, tht, phi in zip(range(num_targets), thts, phis):
-            dist = np.random.uniform(low=1.0, high=self.flight_dome_size)
-            x = dist * math.sin(phi) * math.cos(tht)
-            y = dist * math.sin(phi) * math.sin(tht)
-            z = abs(dist * math.cos(phi))
-            self.targets[i] = np.array([x, y, z])
+    def reset(self, seed=None, options=None):
+        """reset.
 
-        # yaw targets
-        if self.use_yaw_targets:
-            self.yaw_targets = np.random.uniform(
-                low=-math.pi, high=math.pi, size=(num_targets,)
-            )
-
-    def render(self, mode="human"):
-        self.enable_render = True
-
-    def reset(self):
+        Args:
+            seed: seed to pass to the base environment.
+            options:
+        """
         # if we already have an env, disconnect from it
-        if self.env is not None:
+        if hasattr(self, "env"):
             self.env.disconnect()
 
-        # reset step count
-        self.info = {}
-        self.done = False
         self.step_count = 0
+        self.termination = False
+        self.truncation = False
+        self.info = {}
+        self.info["out_of_bounds"] = False
+        self.info["collision"] = False
+        self.info["env_complete"] = False
+
         self.dis_error = -100.0
         self.yaw_error = -100.0
 
@@ -121,6 +136,23 @@ class SimpleWaypointEnv(gym.Env):
         for _ in range(10):
             self.env.step()
 
+        # we sample from polar coordinates to generate linear targets
+        self.targets = np.zeros(shape=(self.num_targets, 3))
+        thts = np.random.uniform(0.0, 2.0 * math.pi, size=(self.num_targets,))
+        phis = np.random.uniform(0.0, 2.0 * math.pi, size=(self.num_targets,))
+        for i, tht, phi in zip(range(self.num_targets), thts, phis):
+            dist = np.random.uniform(low=1.0, high=self.flight_dome_size)
+            x = dist * math.sin(phi) * math.cos(tht)
+            y = dist * math.sin(phi) * math.sin(tht)
+            z = abs(dist * math.cos(phi))
+            self.targets[i] = np.array([x, y, z])
+
+        # yaw targets
+        if self.use_yaw_targets:
+            self.yaw_targets = np.random.uniform(
+                low=-math.pi, high=math.pi, size=(self.num_targets,)
+            )
+
         # if we are rendering, laod in the targets
         if self.enable_render:
             self.target_visual = []
@@ -138,15 +170,19 @@ class SimpleWaypointEnv(gym.Env):
                     rgbaColor=(0, 1 - (i / len(self.target_visual)), 0, 1),
                 )
 
-        return self.compute_state()
+        return self.state, self.info
 
-    def compute_state(self):
-        """This computes the observation as well as the distances to target"""
-        # ang_vel (3/4)
-        # ang_pos (3/4)
-        # lin_vel (3)
-        # lin_pos (3)
-        # body frame distance to targ (3)
+    @property
+    def state(self):
+        """state.
+
+        This returns the observation as well as the distances to target.
+        - ang_vel (vector of 3 values)
+        - ang_pos (vector of 3/4 values)
+        - lin_vel (vector of 3 values)
+        - lin_pos (vector of 3 values)
+        - body_frame distance to target (vector of 3 values)
+        """
         raw_state = self.env.states[0]
 
         # state breakdown
@@ -192,6 +228,8 @@ class SimpleWaypointEnv(gym.Env):
 
     @property
     def target_reached(self):
+        """target_reached.
+        """
         if self.dis_error_scalar < self.goal_reach_distance:
             if self.use_yaw_targets:
                 if self.yaw_error_scalar < self.goal_reach_angle:
@@ -203,35 +241,38 @@ class SimpleWaypointEnv(gym.Env):
 
     @property
     def reward(self):
+        """reward.
+        """
         if len(self.env.getContactPoints()) > 0:
             # collision with ground
             return -100.0
         else:
-            # normal reward
-            error = self.dis_error_scalar
-            if self.use_yaw_targets:
-                error += self.yaw_error_scalar
-            return -error
+            if self.sparse_reward:
+                return 10.0 if self.target_reached else 0.0
+            else:
+                # if not sparse reward, then don't clip it
+                error = self.dis_error_scalar
+                if self.use_yaw_targets:
+                    error += self.yaw_error_scalar
+                return -error
 
-    def compute_done(self):
-        # we were already done
-        if self.done:
-            return True
 
+    def compute_term_trunc(self):
+        """compute_term_trunc.
+        """
         # exceed step count
         if self.step_count > self.max_steps:
-            self.info["done"] = "step_count"
-            return True
+            self.truncation = self.truncation or True
 
         # exceed flight dome
         if np.linalg.norm(self.state[-3:]) > self.flight_dome_size:
-            self.info["done"] = "out_of_range"
-            return True
+            self.info["out_of_bounds"] = True
+            self.termination = self.termination or True
 
         # collision
         if len(self.env.getContactPoints()) > 0:
-            self.info["done"] = "collision"
-            return True
+            self.info["collision"] = True
+            self.termination = self.termination or True
 
         # target reached
         if self.target_reached:
@@ -240,8 +281,8 @@ class SimpleWaypointEnv(gym.Env):
                 self.targets = self.targets[1:]
                 self.yaw_targets = self.yaw_targets[1:]
             else:
-                self.info["done"] = "env_complete"
-                return True
+                self.info["env_complete"] = True
+                self.termination = self.termination or True
 
             # delete the reached target and recolour the others
             if self.enable_render:
@@ -256,25 +297,30 @@ class SimpleWaypointEnv(gym.Env):
                         rgbaColor=(1 - (i / len(self.target_visual)), 0, 0, 1),
                     )
 
-        return False
-
     def step(self, action: np.ndarray):
-        """
-        step the entire simulation
-            output is states, reward, dones
+        """step.
+
+        Args:
+            action (np.ndarray): action
         """
         # unsqueeze the action to be usable in aviary
         action = np.expand_dims(action, axis=0)
 
-        # step env
+        # step through env, the internal env updates a few steps before the outer env
         self.env.set_setpoints(action)
-        while self.env.drones[0].steps % self.env.drones[0].update_ratio != 0:
-            self.env.step()
+        self.env.step()
 
-        # compute state and dones
-        self.state = self.compute_state()
-        self.done = self.compute_done()
+        # compute state and done
+        self.compute_term_trunc()
 
+        # increment step count
         self.step_count += 1
 
-        return self.state, self.reward, self.done, None
+        return self.state, self.reward, self.termination, self.truncation, self.info
+
+    def render(self):
+        """render.
+        """
+        raise AssertionError(
+            "This function is not meant to be called. Apply `render_mode='human'` on environment creation."
+        )
